@@ -3,16 +3,16 @@
  * Copyright (C) 2003  Peter Hanappe and others.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License
- * as published by the Free Software Foundation; either version 2 of
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  * 02110-1301, USA
@@ -46,7 +46,7 @@ struct _fluid_sequencer_t {
 	gboolean useSystemTimer;
 	double scale; // ticks per second
 	fluid_list_t* clients;
-	short clientsID;
+	fluid_seq_id_t clientsID;
 	/* for queue + heap */
 	fluid_evt_entry* preQueue;
 	fluid_evt_entry* preQueueLast;
@@ -67,7 +67,7 @@ struct _fluid_sequencer_t {
 
 /* Private data for clients */
 typedef struct _fluid_sequencer_client_t {
-	short id;
+	fluid_seq_id_t id;
 	char* name;
 	fluid_event_callback_t callback;
 	void* data;
@@ -77,7 +77,7 @@ typedef struct _fluid_sequencer_client_t {
 static short _fluid_seq_queue_init(fluid_sequencer_t* seq, int nbEvents);
 static void _fluid_seq_queue_end(fluid_sequencer_t* seq);
 static short _fluid_seq_queue_pre_insert(fluid_sequencer_t* seq, fluid_event_t * evt);
-static void _fluid_seq_queue_pre_remove(fluid_sequencer_t* seq, short src, short dest, int type);
+static void _fluid_seq_queue_pre_remove(fluid_sequencer_t* seq, fluid_seq_id_t src, fluid_seq_id_t dest, int type);
 static int _fluid_seq_queue_process(void* data, unsigned int msec); // callback from timer
 static void _fluid_seq_queue_insert_entry(fluid_sequencer_t* seq, fluid_evt_entry * evtentry);
 static void _fluid_seq_queue_remove_entries_matching(fluid_sequencer_t* seq, fluid_evt_entry* temp);
@@ -265,7 +265,7 @@ void fluid_seq_dotrace(fluid_sequencer_t* seq, char *fmt, ...) {}
  * Clients can be sources or destinations of events.  Sources don't need to
  * register a callback.
  */
-short
+fluid_seq_id_t
 fluid_sequencer_register_client (fluid_sequencer_t* seq, const char *name,
                                  fluid_event_callback_t callback, void* data)
 {
@@ -281,6 +281,7 @@ fluid_sequencer_register_client (fluid_sequencer_t* seq, const char *name,
 	nameCopy = FLUID_STRDUP(name);
 	if (nameCopy == NULL) {
 		fluid_log(FLUID_PANIC, "sequencer: Out of memory\n");
+		FLUID_FREE(client);
 		return FLUID_FAILED;
 	}
 
@@ -302,7 +303,7 @@ fluid_sequencer_register_client (fluid_sequencer_t* seq, const char *name,
  * @param id Client ID as returned by fluid_sequencer_register_client().
  */
 void
-fluid_sequencer_unregister_client (fluid_sequencer_t* seq, short id)
+fluid_sequencer_unregister_client (fluid_sequencer_t* seq, fluid_seq_id_t id)
 {
 	fluid_list_t *tmp;
 	fluid_event_t* evt;
@@ -357,7 +358,7 @@ fluid_sequencer_count_clients(fluid_sequencer_t* seq)
  * @param index Index of register client
  * @return Client ID or #FLUID_FAILED if not found
  */
-short fluid_sequencer_get_client_id (fluid_sequencer_t* seq, int index)
+fluid_seq_id_t fluid_sequencer_get_client_id (fluid_sequencer_t* seq, int index)
 {
 	fluid_list_t *tmp = fluid_list_nth(seq->clients, index);
 	if (tmp == NULL) {
@@ -429,7 +430,7 @@ fluid_sequencer_client_is_dest(fluid_sequencer_t* seq, int id)
 void
 fluid_sequencer_send_now(fluid_sequencer_t* seq, fluid_event_t* evt)
 {
-	short destID = fluid_event_get_dest(evt);
+	fluid_seq_id_t destID = fluid_event_get_dest(evt);
 
 	/* find callback */
 	fluid_list_t *tmp = seq->clients;
@@ -480,8 +481,8 @@ fluid_sequencer_send_at (fluid_sequencer_t* seq, fluid_event_t* evt,
  * @param type Event type to match or -1 for wildcard (#fluid_seq_event_type)
  */
 void
-fluid_sequencer_remove_events (fluid_sequencer_t* seq, short source,
-                               short dest, int type)
+fluid_sequencer_remove_events (fluid_sequencer_t* seq, fluid_seq_id_t source,
+                               fluid_seq_id_t dest, int type)
 {
 	_fluid_seq_queue_pre_remove(seq, source, dest, type);
 }
@@ -749,7 +750,7 @@ _fluid_seq_queue_pre_insert(fluid_sequencer_t* seq, fluid_event_t * evt)
  * May be called from the main thread (usually) but also recursively
  * from the queue thread, when a callback itself does an insert... */
 static void
-_fluid_seq_queue_pre_remove(fluid_sequencer_t* seq, short src, short dest, int type)
+_fluid_seq_queue_pre_remove(fluid_sequencer_t* seq, fluid_seq_id_t src, fluid_seq_id_t dest, int type)
 {
 	fluid_evt_entry * evtentry = _fluid_seq_heap_get_free(seq->heap);
 	if (evtentry == NULL) {
@@ -981,7 +982,7 @@ _fluid_seq_queue_insert_entry(fluid_sequencer_t* seq, fluid_evt_entry * evtentry
 }
 
 static int
-_fluid_seq_queue_matchevent(fluid_event_t* evt, int templType, short templSrc, short templDest)
+_fluid_seq_queue_matchevent(fluid_event_t* evt, int templType, fluid_seq_id_t templSrc, fluid_seq_id_t templDest)
 {
 	int eventType;
 
@@ -1018,7 +1019,7 @@ _fluid_seq_queue_remove_entries_matching(fluid_sequencer_t* seq, fluid_evt_entry
 {
 	/* we walk everything : this is slow, but that is life */
 	int i, type;
-	short src, dest;
+	fluid_seq_id_t src, dest;
 
 	src = templ->evt.src;
 	dest = templ->evt.dest;
@@ -1203,9 +1204,11 @@ _fluid_seq_queue_send_queued_events(fluid_sequencer_t* seq)
 			_fluid_seq_queue_slide(seq);
 		} /* slide */
 
-
 		/* process queue0[cellNb] */
 		_fluid_seq_queue_send_cell_events(seq, cellNb);
+		
+		/* the current scale may have changed through a callback event */
+		nowTicks = fluid_sequencer_get_tick(seq);
 
 		/* next cell */
 		cellNb++;
